package com.galaxis.manatee.task;

import com.fasterxml.jackson.annotation.JsonProperty;
import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.core.type.TypeReference;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.galaxis.capsule.bo.ChuanyunObject;
import com.galaxis.manatee.constant.ChuanYunConstant;
import com.galaxis.manatee.dao.*;
import com.galaxis.manatee.entity.chuanyun.data.object.*;
import com.galaxis.manatee.entity.chuanyun.dto.ChuanyunSaveDTO;
import com.galaxis.manatee.entity.chuanyun.dto.Filter;
import com.galaxis.manatee.manager.ChuanYunManager;
import lombok.Data;
import lombok.extern.slf4j.Slf4j;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Service;
import org.springframework.util.StringUtils;

import javax.persistence.CascadeType;
import javax.persistence.FetchType;
import javax.persistence.OneToMany;
import javax.persistence.Transient;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

/**
 * @author zcj
 * @version 0.1
 * @date 2020/4/27 5:14 下午
 */
@Service
@Slf4j
public class ChuanyunBasicDataScheduledTask {

    private final Integer pageSize=20;
    private final ChuanYunManager chuanYunManager;
    private final ChuanyunGroupProjectDao chuanyunGroupProjectDao;
    private final ChuanyunUserDao chuanyunUserDao;
    private final ChuanyunSupplierDao chuanyunSupplierDao;
    private final ChuanyunCompanyDao chuanyunCompanyDao;
    private final ChuanyunUserCompanyDao chuanyunUserCompanyDao;
    private final ChuanyunCostDao chuanyunCostDao;
    private final ChuanyunReimbursementSubjectDao chuanyunReimbursementSubjectDao;
    private final ChuanyunProjectTypeDao chuanyunProjectTypeDao;

    public ChuanyunBasicDataScheduledTask(ChuanYunManager chuanYunManager, ChuanyunGroupProjectDao chuanyunGroupProjectDao, ChuanyunUserDao chuanyunUserDao, ChuanyunSupplierDao chuanyunSupplierDao, ChuanyunCompanyDao chuanyunCompanyDao, ChuanyunUserCompanyDao chuanyunUserCompanyDao, ChuanyunCostDao chuanyunCostDao, ChuanyunReimbursementSubjectDao chuanyunReimbursementSubjectDao, ChuanyunProjectTypeDao chuanyunProjectTypeDao) {
        this.chuanYunManager = chuanYunManager;
        this.chuanyunGroupProjectDao = chuanyunGroupProjectDao;
        this.chuanyunUserDao = chuanyunUserDao;
        this.chuanyunSupplierDao = chuanyunSupplierDao;
        this.chuanyunCompanyDao = chuanyunCompanyDao;
        this.chuanyunUserCompanyDao = chuanyunUserCompanyDao;
        this.chuanyunCostDao = chuanyunCostDao;
        this.chuanyunReimbursementSubjectDao = chuanyunReimbursementSubjectDao;
        this.chuanyunProjectTypeDao = chuanyunProjectTypeDao;
    }

    /**
     * 定时将氚云中的数据保存到manatee
     */
    @Scheduled(fixedDelay = 360000L)
    private void getDataFromChuanyun() {

        //获取人员-归属公司信息
        getUserCompany();
        //支出档案类型
        getProjectType();
        //获取报销科目
        getReimbursementSubject();
        //获取成本中心
        getCost();
        //获取用户数据
        getUserList();
        //获取集团项目数据
        getGroupProject();
        //获取公司信息
        getCompany();
        //获取供应商
        getSupplier();

    }

    /**
     * 从氚云中获取用户列表
     */
    private void getUserList() {
        var objectMapper=new ObjectMapper();
        var chuanyunFindAllDTO=chuanYunManager.findAll(ChuanyunUserDO.APP_CODE,ChuanyunUserDO.ACTION_NAME,ChuanyunUserDO.CONTROLLER);
        var totalCount=0;
        try {
            List<ChuanyunUserDO> userList=objectMapper.readValue(chuanyunFindAllDTO.getReturnData().getData(),new TypeReference<>(){});
            userList.forEach(chuanyunUserDO -> {
                if(StringUtils.isEmpty(chuanyunUserDO.getEmployeeNumber())){
                    ChuanYunConstant.USER_EMPLOYEE_NUMBER.put(chuanyunUserDO.getObjectId(),"");
                }else{
                    ChuanYunConstant.USER_EMPLOYEE_NUMBER.put(chuanyunUserDO.getObjectId(),chuanyunUserDO.getEmployeeNumber());
                }
            });
            totalCount=chuanyunUserDao.saveAll(userList).size();
        } catch (Exception e) {
            e.printStackTrace();
        }
        log.info("用户信息保存保存"+totalCount+"工号映射更新完成");
    }

    /**
     * 从氚云获取集团项目信息
     */
    private void getGroupProject(){
        var objectMapper=new ObjectMapper();
        var start=0;
        var totalCount=0L;
        var flag=true;

        while (flag){
            try {
                //从氚云查询数据
                var filter= Filter.instance(start,start+pageSize,true);
                var chuanyunFindAllResponse=chuanYunManager.findAll(ChuanyunGroupProjectDO.SCHEMA_CODE,filter);
                if(chuanyunFindAllResponse.getReturnData()==null){
                    flag=false;
                    continue;
                }
                if(chuanyunFindAllResponse.getReturnData().getTotalCount()>=start+pageSize){
                    start+=pageSize;
                }else{
                    flag=false;
                }
                //转化为POJO
                List<ChuanyunGroupProjectDO> result=objectMapper.convertValue(chuanyunFindAllResponse.getReturnData().getBizObjectArray(), new TypeReference<>(){});
                //获取项目总数
                totalCount=chuanyunFindAllResponse.getReturnData().getTotalCount();
                //保存
                result.forEach(chuanyunGroupProjectDO ->  {
                    var tmpGroupProjectDO=chuanyunGroupProjectDao.findById(chuanyunGroupProjectDO.getObjectId());
                    tmpGroupProjectDO.ifPresent(projectDO -> {
                        chuanyunGroupProjectDO.setIsCreate(projectDO.getIsCreate());
                        chuanyunGroupProjectDao.delete(tmpGroupProjectDO.get());
                    });
                    //新增项目组织信息
//                    List<ChuanyunGroupProjectDetailDO> detaillist = new ArrayList<>();
//                    for (ChuanyunCompanyDO chuanyunCompanyDO : chuanyunCompanyDao.findAll()) {
//                        if (!chuanyunCompanyDO.getCompanyName().equals("浙江凯乐士无锡分公司")&&!chuanyunCompanyDO.getCompanyName().equals("GALAXIS Automation management GmbH")) {
//                            ChuanyunGroupProjectDetailDO chuanyunGroupProjectDetailDO = new ChuanyunGroupProjectDetailDO();
//                            chuanyunGroupProjectDetailDO.setCompanyName(chuanyunCompanyDO.getCompanyName());
//                            chuanyunGroupProjectDetailDO.setType(chuanyunGroupProjectDO.getProjectType());
//                            chuanyunGroupProjectDetailDO.setParentObjectId(chuanyunGroupProjectDO.getObjectId());
//                            detaillist.add(chuanyunGroupProjectDetailDO);
//                        }
//                    }
//                    chuanyunGroupProjectDO.setDetailList(detaillist);
                    //将projectManger和projectOwner替换为主键
                    if(!StringUtils.isEmpty(chuanyunGroupProjectDO.getProjectOwner())){
                        chuanyunGroupProjectDO.setProjectOwner(chuanyunGroupProjectDO.getProjectOwnerObject().getObjectId());
                    }
                    //将项目组成员projectMember替换为主键
                    if (chuanyunGroupProjectDO.getProjectMemberlist()!= null) {
                        var memberlist = new ArrayList<>();
                        var builder = new StringBuilder();
                        for (ChuanyunObject chuanyunObject: chuanyunGroupProjectDO.getProjectMemberObject()) {
                            builder.append(chuanyunObject.getObjectId()+";");
                            memberlist.add(chuanyunObject.getObjectId());
                        }
                        chuanyunGroupProjectDO.setProjectMember(builder.toString());
                        //替换为主键后更新氚云数据表来增加权限
                        chuanyunGroupProjectDO.setProjectMemberlist(memberlist.toArray(new String[memberlist.size()]));
                    }
//                    //如果项目组成员为空，则自动为项目添加所有EBG成员作为项目组成员
//                    if ("EBG".equals(chuanyunGroupProjectDO.getBusinessGroup())&&chuanyunGroupProjectDO.getProjectMemberlist().length==0&&
//                            ("实施".equals(chuanyunGroupProjectDO.getProjectType())||"维保".equals(chuanyunGroupProjectDO.getProjectType())||
//                                    "研发二开".equals(chuanyunGroupProjectDO.getProjectType())||"售后".equals(chuanyunGroupProjectDO.getProjectType()))) {
//                        List<String> projectmember = new ArrayList<>();
//                        for (ChuanyunUserCompanyDO chuanyunUserCompanyDO : chuanyunUserCompanyDao.findAll()) {
//                            if (chuanyunCompanyDao.findByObjectId(chuanyunUserCompanyDO.getCompanyId()).getBusinessGroup().equals("EBG")) {
//                                builder.append(chuanyunUserCompanyDO.getUserId()+";");
//                                projectmember.add(chuanyunUserCompanyDO.getUserId());
//                            }
//                        }
//                        chuanyunGroupProjectDO.setProjectMember(builder.toString());
//                        chuanyunGroupProjectDO.setProjectMemberlist(projectmember.toArray(new String[projectmember.size()]));
//                        log.info("成员已更新");
//                    }

                    //通过更新集团项目中的EBG项目经理、IBG项目经理
                    //TODO 测试结束后需要扩大范围到所有项目
                    GroupProjectUpdateDTO groupProjectUpdateDTO=new GroupProjectUpdateDTO();
                    try {
                        chuanYunManager.update(ChuanyunGroupProjectDO.SCHEMA_CODE,chuanyunGroupProjectDO.getObjectId(),objectMapper.writeValueAsString(groupProjectUpdateDTO));
                    } catch (JsonProcessingException e) {
                        e.printStackTrace();
                    }
                    groupProjectUpdateDTO=new GroupProjectUpdateDTO(chuanyunGroupProjectDO);
                    //保证项目经理信息更新成功
                    boolean updateFlag=false;
                    while (!updateFlag){
                        try {
                            var groupProjectUpdateString=objectMapper.writeValueAsString(groupProjectUpdateDTO);
                            ChuanyunSaveDTO chuanyunSaveDTO=chuanYunManager.update(ChuanyunGroupProjectDO.SCHEMA_CODE,chuanyunGroupProjectDO.getObjectId(),groupProjectUpdateString);
                            if(chuanyunSaveDTO.getSuccessful()){
                                log.info(chuanyunGroupProjectDO.getObjectId()+"更新项目经理成功"+groupProjectUpdateString);
                                updateFlag=true;
                            }else{
                                log.warn(chuanyunGroupProjectDO.getObjectId()+"更新项目经理失败"+groupProjectUpdateString);
                                log.warn(chuanyunGroupProjectDO+"");
                                Thread.sleep(5000L);
                            }
                        } catch (Exception e) {
                            e.printStackTrace();
                        }
                    }


                });
                chuanyunGroupProjectDao.saveAll(result);
            }catch (Exception e){
                log.error(e.getMessage());
                e.printStackTrace();
            }
        }
        log.info("集团项目信息保存保存"+totalCount+"项目映射更新完成");
    }

    /**
     * 获取供应商信息
     */
    private void getSupplier(){
        var objectMapper=new ObjectMapper();
        var start=0;
        var totalCount=0L;
        var flag=true;
        while (flag){
            try {
                //从氚云查询数据
                var filter= Filter.instance(start,start+pageSize,true);
                var chuanyunFindAllResponse=chuanYunManager.findAll(ChuanyunSupplierDO.SCHEMA_CODE,filter);
                if(chuanyunFindAllResponse.getReturnData()==null){
                    flag=false;
                    continue;
                }
                if(chuanyunFindAllResponse.getReturnData().getTotalCount()>=start+pageSize){
                    start+=pageSize;
                }else{
                    flag=false;
                }
                //转化为POJO
                List<ChuanyunSupplierDO> result=objectMapper.convertValue(chuanyunFindAllResponse.getReturnData().getBizObjectArray(), new TypeReference<>(){});
                //获取项目总数
                totalCount=chuanyunFindAllResponse.getReturnData().getTotalCount();
                //保存
                chuanyunSupplierDao.saveAll(result);
            }catch (Exception e){
                log.error(e.getMessage());
                e.printStackTrace();
            }
        }
        log.info("供应商信息保存保存"+totalCount);
    }

    /**
     * 更新支出档案类型
     */
    public void getProjectType() {
        var objectMapper=new ObjectMapper();
        var start=0;
        var totalCount=0L;
        var flag=true;
        while (flag) {
            try {
                //从氚云查询数据
                var filter = Filter.instance(start, start + pageSize, true);
                var chuanyunFindAllResponse = chuanYunManager.findAll(ChuanyunProjectTypeDO.SCHEMA_CODE, filter);
                if (chuanyunFindAllResponse.getReturnData() == null) {
                    flag = false;
                    continue;
                }
                if (chuanyunFindAllResponse.getReturnData().getTotalCount() >= start + pageSize) {
                    start += pageSize;
                } else {
                    flag = false;
                }
                //转化为POJO
                List<ChuanyunProjectTypeDO> result = objectMapper.convertValue(chuanyunFindAllResponse.getReturnData().getBizObjectArray(), new TypeReference<>(){});
                //获取项目总数
                totalCount=chuanyunFindAllResponse.getReturnData().getTotalCount();
                //保存
                chuanyunProjectTypeDao.saveAll(result);
            }catch (Exception e) {
                log.error(e.getMessage());
                e.printStackTrace();
            }
            log.info("支出档案类型"+totalCount);
        }
    }

    /**
     * 更新公司信息
     */
    private void getCompany(){
        var objectMapper=new ObjectMapper();
        var start=0;
        var totalCount=0L;
        var flag=true;
        while (flag){
            try {
                //从氚云查询数据
                var filter= Filter.instance(start,start+pageSize,true);
                var chuanyunFindAllResponse=chuanYunManager.findAll(ChuanyunCompanyDO.SCHEMA_CODE,filter);
                if(chuanyunFindAllResponse.getReturnData()==null){
                    flag=false;
                    continue;
                }
                if(chuanyunFindAllResponse.getReturnData().getTotalCount()>=start+pageSize){
                    start+=pageSize;
                }else{
                    flag=false;
                }
                //转化为POJO
                List<ChuanyunCompanyDO> result=objectMapper.convertValue(chuanyunFindAllResponse.getReturnData().getBizObjectArray(), new TypeReference<>(){});
                //获取项目总数
                totalCount=chuanyunFindAllResponse.getReturnData().getTotalCount();
                //保存
                chuanyunCompanyDao.saveAll(result);
            }catch (Exception e){
                log.error(e.getMessage());
                e.printStackTrace();
            }
        }
        log.info("公司信息保存保存"+totalCount);
    }

    /**
     * 更新成本中心
     */
    private void getCost(){
        var objectMapper=new ObjectMapper();
        var start=0;
        var totalCount=0L;
        var flag=true;
        while (flag){
            try {
                //从氚云查询数据
                var filter= Filter.instance(start,start+pageSize,true);
                var chuanyunFindAllResponse=chuanYunManager.findAll(ChuanyunCostDO.SCHEMA_CODE,filter);
                if(chuanyunFindAllResponse.getReturnData()==null){
                    flag=false;
                    continue;
                }
                if(chuanyunFindAllResponse.getReturnData().getTotalCount()>=start+pageSize){
                    start+=pageSize;
                }else{
                    flag=false;
                }
                //转化为POJO
                List<ChuanyunCostDO> result=objectMapper.convertValue(chuanyunFindAllResponse.getReturnData().getBizObjectArray(), new TypeReference<>(){});
                //获取项目总数
                totalCount=chuanyunFindAllResponse.getReturnData().getTotalCount();
                //保存
                chuanyunCostDao.saveAll(result);
            }catch (Exception e){
                log.error(e.getMessage());
                e.printStackTrace();
            }
        }
        log.info("成本中心保存保存"+totalCount);
    }

    /**
     * 获取人员-归属公司信息
     */
    private void getUserCompany(){
        var objectMapper=new ObjectMapper();
        var start=0;
        var totalCount=0L;
        var flag=true;
        while (flag){
            try {
                //从氚云查询数据
                var filter= Filter.instance(start,start+pageSize,true);
                var chuanyunFindAllResponse=chuanYunManager.findAll(ChuanyunUserCompanyDO.SCHEMA_CODE,filter);
                if(chuanyunFindAllResponse.getReturnData()==null){
                    flag=false;
                    continue;
                }
                if(chuanyunFindAllResponse.getReturnData().getTotalCount()>=start+pageSize){
                    start+=pageSize;
                }else{
                    flag=false;
                }
                //转化为POJO
                List<ChuanyunUserCompanyDO> result=objectMapper.convertValue(chuanyunFindAllResponse.getReturnData().getBizObjectArray(), new TypeReference<>(){});
                //获取项目总数
                totalCount=chuanyunFindAllResponse.getReturnData().getTotalCount();
                //保存
                result.forEach(chuanyunUserCompanyDO -> {
                    try {
                        if(chuanyunUserCompanyDO.getUser()!=null){
                            chuanyunUserCompanyDO.setUserId(chuanyunUserCompanyDO.getUser().getObjectId());
                        }
                    }catch (Exception exception){
                        log.warn(chuanyunUserCompanyDO+"");
                        log.warn(exception.getMessage());
                    }
                    ChuanYunConstant.USER_DEPARTMENT.put(chuanyunUserCompanyDO.getUserId(),chuanyunUserCompanyDO.getDepartmentObject().getObjectId());
                });
                chuanyunUserCompanyDao.saveAll(result);
            }catch (Exception e){
                log.error(e.getMessage());
                e.printStackTrace();
            }
        }
        log.info("人员归属公司信息保存保存"+totalCount);
    }

    /**
     * 获取报销科目
     */
    private void getReimbursementSubject(){
        var objectMapper=new ObjectMapper();
        var start=0;
        var totalCount=0L;
        var flag=true;
        while (flag){
            try {
                //从氚云查询数据
                var filter= Filter.instance(start,start+pageSize,true);
                var chuanyunFindAllResponse=chuanYunManager.findAll(ChuanyunReimbursementSubjectDO.SCHEMA_CODE,filter);
                if(chuanyunFindAllResponse.getReturnData()==null){
                    flag=false;
                    continue;
                }
                if(chuanyunFindAllResponse.getReturnData().getTotalCount()>=start+pageSize){
                    start+=pageSize;
                }else{
                    flag=false;
                }
                //转化为POJO
                List<ChuanyunReimbursementSubjectDO> result=objectMapper.convertValue(chuanyunFindAllResponse.getReturnData().getBizObjectArray(), new TypeReference<>(){});
                //获取项目总数
                totalCount=chuanyunFindAllResponse.getReturnData().getTotalCount();
                //保存
                chuanyunReimbursementSubjectDao.saveAll(result);
            }catch (Exception e){
                log.error(e.getMessage());
                e.printStackTrace();
            }
        }
        log.info("报销科目信息保存保存"+totalCount);
    }

    /**
     * 用于刷新项目经理权限的对象
     */
    @Data
    class GroupProjectUpdateDTO{
        /**
         * 项目总监
         * U9项目责任人
         */
        @JsonProperty(value = "F0000006")
        private String projectOwner;

        /**
         * IBG项目经理
         */
        @JsonProperty("F0000012")
        private String ibgProjectManager;

        /**
         * EBG项目经理
         */
        @JsonProperty("F0000011")
        private String ebgProjectManager;

        /**
         * 项目组成员（前端传数据用）
         */
        @JsonProperty("F0000034")
        private String[] projectMemberlist;

        /**
         * 项目组织信息
         */
        @OneToMany(cascade = CascadeType.ALL, orphanRemoval = true,fetch = FetchType.EAGER)
        @JsonProperty("D001789Fd992bf8fbe114c9babe0f98cf676f8a2")
        private List<ChuanyunGroupProjectDetailDO> detailList;



        public GroupProjectUpdateDTO() {
        }

        public GroupProjectUpdateDTO(ChuanyunGroupProjectDO chuanyunGroupProjectDO) {
            if(null!=chuanyunGroupProjectDO){
                if(null!=chuanyunGroupProjectDO.getProjectOwnerObject()){
                    this.projectOwner=chuanyunGroupProjectDO.getProjectOwnerObject().getObjectId();
                }
                if(null!=chuanyunGroupProjectDO.getIbgProjectManagerObject()){
                    this.ibgProjectManager=chuanyunGroupProjectDO.getIbgProjectManagerObject().getObjectId();
                }
                if(null!=chuanyunGroupProjectDO.getEbgProjectManagerObject()){
                    this.ebgProjectManager=chuanyunGroupProjectDO.getEbgProjectManagerObject().getObjectId();
                }
                if(null!=chuanyunGroupProjectDO.getProjectMemberlist()){
                    this.projectMemberlist=chuanyunGroupProjectDO.getProjectMemberlist();
                }
                if(null!=chuanyunGroupProjectDO.getDetailList()){
                    this.detailList=chuanyunGroupProjectDO.getDetailList();
                }
            }
        }
    }
}
