package com.galaxis.manatee.task;

import com.fasterxml.jackson.core.type.TypeReference;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.galaxis.manatee.constant.ChuanYunConstant;
import com.galaxis.manatee.dao.*;
import com.galaxis.manatee.entity.chuanyun.data.object.ChuanyunGroupProjectDO;
import com.galaxis.manatee.entity.chuanyun.data.object.ChuanyunHolidaySalaryRuleDO;
import com.galaxis.manatee.entity.chuanyun.data.object.ChuanyunLogDO;
import com.galaxis.manatee.entity.chuanyun.dto.ChuanyunSaveDTO;
import com.galaxis.manatee.entity.chuanyun.dto.Filter;
import com.galaxis.manatee.manager.ChuanYunManager;
import lombok.extern.slf4j.Slf4j;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Service;
import org.springframework.util.StringUtils;

import java.util.ArrayList;
import java.util.List;

/**
 * @author zcj
 * @version 0.1
 * @date 2020/5/22 11:33 下午
 */
@Slf4j
@Service
public class ChuanyunHumanResourceScheduledTask {

    private final Integer pageSize = 100;
    private final ChuanYunManager chuanYunManager;
    private final ChuanyunLogDao chuanyunLogDao;
    private final ChuanyunGroupProjectDao chuanyunGroupProjectDao;
    private final ChuanyunHolidaySalaryRuleDao chuanyunHolidaySalaryRuleDao;
    private final ChuanyunUserCompanyDao chuanyunUserCompanyDao;
    private final ChuanyunCompanyDao chuanyunCompanyDao;

    public ChuanyunHumanResourceScheduledTask(ChuanYunManager chuanYunManager, ChuanyunLogDao chuanyunLogDao, ChuanyunGroupProjectDao chuanyunGroupProjectDao, ChuanyunHolidaySalaryRuleDao chuanyunHolidaySalaryRuleDao, ChuanyunUserCompanyDao chuanyunUserCompanyDao, ChuanyunCompanyDao chuanyunCompanyDao) {
        this.chuanYunManager = chuanYunManager;
        this.chuanyunLogDao = chuanyunLogDao;
        this.chuanyunGroupProjectDao = chuanyunGroupProjectDao;
        this.chuanyunHolidaySalaryRuleDao = chuanyunHolidaySalaryRuleDao;
        this.chuanyunUserCompanyDao = chuanyunUserCompanyDao;
        this.chuanyunCompanyDao = chuanyunCompanyDao;
    }

    /**
     * 每天3点更新日志记录
     */
//    @Scheduled(fixedDelay = 1000000000L)
    @Scheduled(cron = "0 0 3 * * *")
//    @Scheduled(fixedDelay = 360000L)
    private void updateLog() {
        var objectMapper = new ObjectMapper();
        var start = 0;
        var totalCount = 0L;
        var flag = true;
        List<String> matchersString=new ArrayList<>();
        matchersString.add("Status_2,1");
        while (flag) {
            try {
                //从氚云查询数据
                var filter= Filter.instance(start,start+pageSize,true,Filter.AND,matchersString);
                var chuanyunFindAllResponse = chuanYunManager.findAll(ChuanyunLogDO.SCHEMA_CODE, filter);
                if (chuanyunFindAllResponse.getReturnData() == null) {
                    flag = false;
                    continue;
                }
                if (chuanyunFindAllResponse.getReturnData().getTotalCount() >= start + pageSize) {
                    start += pageSize;
                } else {
                    flag = false;
                }
                //转化为POJO
                List<ChuanyunLogDO> result = objectMapper.convertValue(chuanyunFindAllResponse.getReturnData().getBizObjectArray(), new TypeReference<>() {
                });
                result.forEach(chuanyunLogDO -> {
                    if(chuanyunLogDO.getDayLogList()==null||chuanyunLogDO.getDayLogList().size()==0){
                        chuanyunLogDO.setDayLogList(new ArrayList<>());
                        log.warn(chuanyunLogDO+"");
                        log.warn("异常数据");
                    }else{
                        //如果有重复就删除
                        chuanyunLogDao.findById(chuanyunLogDO.getObjectId()).ifPresent(toDelete -> {
                            log.info("删除" + toDelete.getObjectId());
                            chuanyunLogDao.delete(toDelete);
                        });
                        //根据projectId获取code,如果是生产项目，则不用处理
                        //针对前期测试数据，可能出现日志为空的情况，跳过不处理
                        chuanyunLogDO.getDayLogList().forEach(chuanyunDayLog -> {
                            try{
                                if(!"生产项目".equals(chuanyunDayLog.getProjectType())&& StringUtils.isEmpty(chuanyunDayLog.getProjectId())){
                                    log.info(chuanyunLogDO+"");
                                }else if(!"生产项目".equals(chuanyunDayLog.getProjectType())){
                                    ChuanyunGroupProjectDO chuanyunGroupProjectDO=chuanyunGroupProjectDao.findById(chuanyunDayLog.getProjectId()).orElseThrow();
                                    chuanyunDayLog.setProjectCode(chuanyunGroupProjectDO.getProjectCode());
                                }
                            }catch (Exception e){
                                e.printStackTrace();
                                log.warn(chuanyunDayLog+"");
                            }
                        });
                        //判断所属部门是否正确
                        chuanyunUserCompanyDao.findById(chuanyunLogDO.getUserCompanyId()).ifPresent(chuanyunUserCompanyDO -> {
                            String ownerDepartmentId = chuanyunUserCompanyDO.getDepartment();
                            if (!chuanyunLogDO.getOwnerDepartmentId().equals(ownerDepartmentId)) {
                                chuanyunLogDO.setOwnerDepartmentId(ChuanYunConstant.USER_DEPARTMENT.get(chuanyunUserCompanyDO.getUserId()).toString());
                                boolean updateFlag=false;
                                while (!updateFlag) {
                                    try {
                                        var groupProjectUpdateString = objectMapper.writeValueAsString(chuanyunLogDO);
                                        ChuanyunSaveDTO chuanyunSaveDTO = chuanYunManager.update(ChuanyunLogDO.SCHEMA_CODE, chuanyunLogDO.getObjectId(), groupProjectUpdateString);
                                        if (chuanyunSaveDTO.getSuccessful()) {
                                            log.info(chuanyunLogDO.getObjectId() + "更新人员所属部门成功" + groupProjectUpdateString);
                                            updateFlag = true;
                                        } else {
                                            log.warn(chuanyunLogDO.getObjectId() + "更新人员所属部门失败" + groupProjectUpdateString);
                                            log.warn(chuanyunLogDO + "");
                                            Thread.sleep(5000L);
                                        }
                                    } catch (Exception e) {
                                        e.printStackTrace();
                                    }
                                }
                            }
                        });
                    }
                    //处理项目userId
                    if(chuanyunLogDO.getUserObject()!=null){
                        chuanyunLogDO.setUserId(chuanyunLogDO.getUserObject().getObjectId());
                    }
                });
                //获取项目总数
                totalCount = chuanyunFindAllResponse.getReturnData().getTotalCount();
                //保存
                chuanyunLogDao.saveAll(result);
            } catch (Exception e) {
                log.error(e.getMessage());
                e.printStackTrace();
            }
        }
        log.info("日志信息保存" + totalCount);
    }

    @Scheduled(fixedDelay = 1000000000L)
    private void getHolidaySalaryRule() {
        var objectMapper = new ObjectMapper();
        var start = 0;
        var totalCount = 0L;
        var flag = true;
        while (flag) {
            try {
                //从氚云查询数据
                var filter = Filter.instance(start, start + pageSize, true);
                var chuanyunFindAllResponse = chuanYunManager.findAll(ChuanyunHolidaySalaryRuleDO.SCHEMA_CODE, filter);
                if (chuanyunFindAllResponse.getReturnData() == null) {
                    flag = false;
                    continue;
                }
                if (chuanyunFindAllResponse.getReturnData().getTotalCount() >= start + pageSize) {
                    start += pageSize;
                } else {
                    flag = false;
                }
                //转化为POJO
                List<ChuanyunHolidaySalaryRuleDO> result = objectMapper.convertValue(chuanyunFindAllResponse.getReturnData().getBizObjectArray(), new TypeReference<>() {});
                //获取项目总数
                totalCount=chuanyunFindAllResponse.getReturnData().getTotalCount();
                //保存数据
                chuanyunHolidaySalaryRuleDao.saveAll(result);
            }catch (Exception e) {
                log.error(e.getMessage());
                e.printStackTrace();
            }
        }
        log.info("法定假日薪资规则同步"+totalCount);
    }
}
